/*
 * Copyright (C) 2023-2024
 * Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package logging

import (
	"antragsraum-proxy/internal/config"
	"fmt"
	"log"
	"os"
	"sync"
)

const (
	logLevelNumError = iota
	logLevelNumWarning
	logLevelNumInfo
	logLevelNumDebug
	LogLevelError   = "ERROR"
	LogLevelWarning = "WARN"
	LogLevelInfo    = "INFO"
	LogLevelDebug   = "DEBUG"
)

type Logger struct {
	BaseLogger *log.Logger
	level      int
}

var (
	logger *Logger
	once   sync.Once
)

func GetLogger() *Logger {
	initializeLogger()

	return logger
}

func initializeLogger() {
	once.Do(func() {
		logger = &Logger{
			BaseLogger: log.New(os.Stdout, "", log.LstdFlags),
			level:      getLoggingLevel(),
		}
	})
}

func getLoggingLevel() int {
	logLevelMap := map[string]int{
		LogLevelError:   logLevelNumError,
		LogLevelWarning: logLevelNumWarning,
		LogLevelInfo:    logLevelNumInfo,
		LogLevelDebug:   logLevelNumDebug,
	}

	conf := config.LoadConfig()
	logLevel, exists := logLevelMap[conf.Logging.Level]
	if !exists {
		logLevel = logLevelMap[LogLevelInfo]
	}

	return logLevel
}

func (l *Logger) log(level int, prefix, format string, v ...any) {
	if l.level >= level {
		l.BaseLogger.Println(fmt.Sprintf("%v: %v", prefix, fmt.Sprintf(format, v...)))
	}
}

func (l *Logger) Fatal(format string, v ...any) {
	l.BaseLogger.Fatalln(fmt.Sprintf("FATAL: "+format, v...))
}

func (l *Logger) Error(format string, v ...any) {
	l.log(logLevelNumError, "ERROR", format, v...)
}

func (l *Logger) Warning(format string, v ...any) {
	l.log(logLevelNumWarning, "WARNING", format, v...)
}

func (l *Logger) Info(format string, v ...any) {
	l.log(logLevelNumInfo, "INFO", format, v...)
}

func (l *Logger) Debug(format string, v ...any) {
	l.log(logLevelNumDebug, "DEBUG", format, v...)
}
