/*
 * Copyright (C) 2023-2024
 * Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package server

import (
	"antragsraum-proxy/internal/mock"
	"bytes"
	"github.com/stretchr/testify/assert"
	"io"
	"mime/multipart"
	"net/http"
	"os"
	"testing"
)

func TestRegisterHomeEndpoint(t *testing.T) {
	SetUpHttpGateway()

	resp, err := http.Get("http://localhost:8082/")

	assert.NoError(t, err)
	assert.Equal(t, http.StatusOK, resp.StatusCode)
}

func TestRegisterAntragraumEndpoints(t *testing.T) {
	t.Run("should find rueckfragen", func(t *testing.T) {
		mock.SetUpGrpcServer()
		SetUpGrpcRouter()
		SetUpHttpGateway()

		jsonData := []byte(`{"postfachId": "testPostfachId"}`)
		req, err := http.NewRequest("POST", "http://localhost:8082/api/v1/rueckfragen", bytes.NewBuffer(jsonData))
		req.Header.Set(GrpcAddressHeader, "localhost")

		client := &http.Client{}
		resp, err := client.Do(req)

		assert.NoError(t, err)
		assert.Equal(t, http.StatusOK, resp.StatusCode)
	})

	t.Run("should get rueckfrage", func(t *testing.T) {
		mock.SetUpGrpcServer()
		SetUpGrpcRouter()
		SetUpHttpGateway()

		jsonData := []byte(`{"id": "testId"}`)
		req, err := http.NewRequest("POST", "http://localhost:8082/api/v1/rueckfrage", bytes.NewBuffer(jsonData))
		req.Header.Set(GrpcAddressHeader, "localhost")

		client := &http.Client{}
		resp, err := client.Do(req)

		assert.NoError(t, err)
		assert.Equal(t, http.StatusOK, resp.StatusCode)
	})

	t.Run("should send answer", func(t *testing.T) {
		mock.SetUpGrpcServer()
		SetUpGrpcRouter()
		SetUpHttpGateway()

		jsonData := []byte(`{}`)
		req, err := http.NewRequest("POST", "http://localhost:8082/api/v1/antworten", bytes.NewBuffer(jsonData))
		req.Header.Set(GrpcAddressHeader, "localhost")

		client := &http.Client{}
		resp, err := client.Do(req)

		assert.NoError(t, err)
		assert.Equal(t, http.StatusOK, resp.StatusCode)
	})

	t.Run("should get attachment metadata", func(t *testing.T) {
		mock.SetUpGrpcServer()
		SetUpGrpcRouter()
		SetUpHttpGateway()

		jsonData := []byte(`{"fileId": "testFileId"}`)
		req, err := http.NewRequest("POST", "http://localhost:8082/api/v1/file/metadata", bytes.NewBuffer(jsonData))
		req.Header.Set(GrpcAddressHeader, "localhost")

		client := &http.Client{}
		resp, err := client.Do(req)

		assert.NoError(t, err)
		assert.Equal(t, http.StatusOK, resp.StatusCode)
	})
}

func TestRegisterCommandEndpoints(t *testing.T) {
	mock.SetUpGrpcServer()
	SetUpGrpcRouter()
	SetUpHttpGateway()

	jsonData := []byte(``)
	req, err := http.NewRequest("GET", "http://localhost:8082/api/v1/command/testId", bytes.NewBuffer(jsonData))
	req.Header.Set(GrpcAddressHeader, "localhost")

	client := &http.Client{}
	resp, err := client.Do(req)

	assert.NoError(t, err)
	assert.Equal(t, http.StatusOK, resp.StatusCode)
}

func TestRegisterUploadAttachmentEndpoint(t *testing.T) {
	mock.SetUpGrpcServer()
	SetUpHttpGateway()

	var requestBody bytes.Buffer
	writer := multipart.NewWriter(&requestBody)

	file, err := os.Open("testdata/dummy.pdf")
	assert.NoError(t, err)
	defer file.Close()

	part, err := writer.CreateFormFile("file", "dummy.pdf")
	assert.NoError(t, err)

	_, err = io.Copy(part, file)
	assert.NoError(t, err)

	err = writer.Close()
	assert.NoError(t, err)

	req, err := http.NewRequest("POST", "http://localhost:8082/api/v1/file", &requestBody)
	req.Header.Set("Content-Type", writer.FormDataContentType())
	req.Header.Set(GrpcAddressHeader, "localhost")

	client := &http.Client{}
	resp, err := client.Do(req)

	assert.NoError(t, err)
	assert.Equal(t, http.StatusOK, resp.StatusCode)
}

func TestRegisterGetAttachmentContentEndpoint(t *testing.T) {
	mock.SetUpGrpcServer()
	SetUpHttpGateway()

	jsonData := []byte(`{}`)
	req, err := http.NewRequest("POST", "http://localhost:8082/api/v1/file/content", bytes.NewBuffer(jsonData))
	req.Header.Set(GrpcAddressHeader, "localhost")

	client := &http.Client{}
	resp, err := client.Do(req)

	assert.NoError(t, err)
	assert.Equal(t, http.StatusOK, resp.StatusCode)
}

func TestErrorHandler(t *testing.T) {
	assertErroneousRequest := func(jsonData []byte, expectedStatusCode int) {
		mock.SetUpGrpcServer()
		SetUpGrpcRouter()
		SetUpHttpGateway()

		req, err := http.NewRequest("POST", "http://localhost:8082/api/v1/rueckfragen", bytes.NewBuffer(jsonData))
		req.Header.Set(GrpcAddressHeader, "localhost")

		client := &http.Client{}
		resp, err := client.Do(req)

		assert.NoError(t, err)
		assert.Equal(t, expectedStatusCode, resp.StatusCode)
	}

	t.Run("should have bad request error", func(t *testing.T) {
		jsonData := []byte(`{}`)

		assertErroneousRequest(jsonData, http.StatusBadRequest)
	})

	t.Run("should have conflict error", func(t *testing.T) {
		jsonData := []byte(`{"postfachId": "conflictId"}`)

		assertErroneousRequest(jsonData, http.StatusConflict)
	})

	t.Run("should have unavailable error", func(t *testing.T) {
		jsonData := []byte(`{"postfachId": "unavailableId"}`)

		assertErroneousRequest(jsonData, http.StatusServiceUnavailable)
	})

	t.Run("should have internal server error", func(t *testing.T) {
		jsonData := []byte(`{"postfachId": "erroneousId"}`)

		assertErroneousRequest(jsonData, http.StatusInternalServerError)
	})
}
